package org.processmining.dataawarecnetminer.mining.classic;

import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.dataawarecnetminer.model.DependencyRelation;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.models.causalgraph.Relation;

public class AcceptedConnectedHeuristic {

	public static void connectAcceptedTasks(EventRelationStorage eventStore,
			final Map<Relation, DependencyRelation> graphRelations, double relativeToBestRatio) {
		boolean hasUnconnected = true;
		while (hasUnconnected) {
			hasUnconnected = false;
			for (Relation relation : graphRelations.keySet()) {
				if (!hasSource(eventStore, graphRelations.keySet(), relation.getSource())) {
					// add strongest incoming
					Relation strongestCause = StrongestRelationsHeuristic.findStrongestCause(eventStore,
							relation.getSource());
					if (strongestCause != null) {
						hasUnconnected = true;
						graphRelations.put(strongestCause, eventStore.getDependencyRelation(strongestCause));
						graphRelations.putAll(StrongestRelationsHeuristic.findRelativeToBestCauses(eventStore,
								strongestCause, relativeToBestRatio));
					}
				}
				if (!hasTarget(eventStore, graphRelations.keySet(), relation.getTarget())) {
					// add strongest outgoing
					Relation strongestFollower = StrongestRelationsHeuristic.findStrongestFollower(eventStore,
							relation.getTarget());
					if (strongestFollower != null) {
						hasUnconnected = true;
						graphRelations.put(strongestFollower, eventStore.getDependencyRelation(strongestFollower));
						graphRelations.putAll(StrongestRelationsHeuristic.findRelativeToBestFollowers(eventStore,
								strongestFollower, relativeToBestRatio));
					}
				}
				if (hasUnconnected) {
					break;
				}
			}
		}
	}

	private static boolean hasTarget(EventRelationStorage eventStore, Set<Relation> graph, XEventClass node) {
		if (node.equals(eventStore.getEndEventClass())) {
			return true; // end does not need a target
		}
		for (Relation relation : graph) {
			if (!relation.getSource().equals(relation.getTarget()) && relation.getSource().equals(node)) {
				return true;
			}
		}
		return false;
	}

	private static boolean hasSource(EventRelationStorage eventStore, Set<Relation> graph, XEventClass node) {
		if (node.equals(eventStore.getStartEventClass())) {
			return true; // end does not need a target
		}
		for (Relation relation : graph) {
			if (!relation.getSource().equals(relation.getTarget()) && relation.getTarget().equals(node)) {
				return true;
			}
		}
		return false;
	}

}
