package org.processmining.dataawarecnetminer.mining.classic;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XLog;
import org.processmining.dataawarecnetminer.extension.dependencies.DependencyHeuristicConfig;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;

public final class HeuristicsCausalGraphMiner {

	private final EventRelationStorage eventRelations;
	private final HeuristicsCausalGraphBuilder heuristicsGraphBuilder;

	public HeuristicsCausalGraphMiner(XLog log, XEventClassifier classifier) {
		this(EventRelationStorage.Factory.createEventRelations(log, classifier));
	}

	public HeuristicsCausalGraphMiner(EventRelationStorage eventRelations) {
		this.eventRelations = eventRelations;
		this.heuristicsGraphBuilder = new HeuristicsCausalGraphBuilder(getEventRelations());
	}

	public DependencyAwareCausalGraph mineCausalGraph() {
		return heuristicsGraphBuilder.build();
	}
	
	public void setHeuristicsConfig(DependencyHeuristicConfig config) {
		heuristicsGraphBuilder.setConfig(config);
	}

	public DependencyHeuristicConfig getHeuristicsConfig() {
		return heuristicsGraphBuilder.getConfig();
	}

	public EventRelationStorage getEventRelations() {
		return eventRelations;
	}

}